/*  This file is part of  e6  (EndNote's Sibling In Xlib)
 *
 *  The code contained here handles text input into previously prepared windows
 *  relating to a reference type.
 *
 *  Coded by:       Ross Maloney
 *  May 2011:       Initial coding
 *  May 2012:       Revision 1
 *  January 2013:   Enter in selection window special case
 *  September 2014  Special handline of multi-line Note field dataStore[26]
 *  June 2018       Note fieldd process changed to Annotation field dataStore[65]
 */

#include  <X11/Xlib.h>
#include  <X11/Xutil.h>
#include  <X11/keysym.h>
#include  <X11/keysymdef.h>
#include  <stdio.h>
#include  "e6x11Ext.h"       /* X11 related common parts of e6 */
#include  "e6dataExt.h"      /* Common parts for processing data */

#define  LINESIZE charinc*NO

/* This routine gets each character typed from the keyboard, interprets it,
 * stores that character, and then prints it on the window in which it was
 * typed - one character per call of this function.  However, if the input
 * field is that of Annotation, then the keyNote() routine is used instead
 * of this since multi line text is processed slightly differently.
 */

void keyEntry(XKeyEvent *keyevent, int index)
{
  int     count, k, i;
  KeySym  keysymbol;
  XComposeStatus   cs;
  char    bytes[3];
  void    searchShow(void);   /* in file findshow.c */

  count = XLookupString(keyevent, bytes, 3, &keysymbol, &cs);
  i = index - 1000;  /* all text input fields into here are biased by 1000 */

                      /* Determine action to be performed for each character */
  switch (count) {
  case 0:             /* Non-printable key */
   switch ( keysymbol )  {
   case  XK_KP_Left:           /* numeric keypad Left arrow */
   case  XK_Left:              /* Left arrow */
     if ( dataStore[i].current <= 0 )
       XBell(e6display, 20);
     else  {
       dataStore[i].current--;
       XClearWindow(e6display, dataStore[i].id);
       XCopyArea(e6display, insertMarker, dataStore[i].id, GC1,
                 0, 0, 6, 16, dataStore[i].current * charinc, 2);
       XDrawString(e6display, dataStore[i].id, GC1, 0, 15,
	           dataStore[i].array, dataStore[i].end);
     }
     break;
   case  XK_Right:             /* Right arrow */
   case  XK_KP_Right:          /* numeric keypad Right arrow */
     if ( dataStore[i].current == dataStore[i].end )
       XBell(e6display, 20);
     else  {
       dataStore[i].current++;
       XClearWindow(e6display, dataStore[i].id);
       XCopyArea(e6display, insertMarker, dataStore[i].id, GC1,
                 0, 0, 6, 16, dataStore[i].current * charinc, 2);
       XDrawString(e6display, dataStore[i].id, GC1, 0, 15,
	           dataStore[i].array, dataStore[i].end);
     }
     break;
   case  XK_Up:                /* Up arrow */
   case  XK_KP_Up:             /* numeric keypad Up arrow */
     k = dataStore[i].before;
     XWarpPointer(e6display, None, dataStore[k].id,
                  0, 0, 0, 0, dataStore[k].end*charinc, 15); 
     break;
   case  XK_Down:             /* Down arrow */
   case  XK_KP_Down:          /* numeric keypad Down arrow */
     k = dataStore[i].next;
     XWarpPointer(e6display, None, dataStore[k].id,
                  0, 0, 0, 0, dataStore[k].end*charinc, 15);
     break;
   }
   break;
  case 1:             /* Printable key */
    switch ( bytes[0] )  {
    case  8:                 /* Backspace */
      if ( dataStore[i].current <= 0 )  {
        dataStore[i].current = 0;
	dataStore[i].end = 0;
      }
      else  {
        XClearWindow(e6display, dataStore[i].id);
	dataStore[i].current--;
	for (k=dataStore[i].current; k<dataStore[i].end; k++)
	  dataStore[i].array[k] = dataStore[i].array[k+1];
//	dataStore[i].current--;
	dataStore[i].end--;
        XCopyArea(e6display, insertMarker, dataStore[i].id, GC1,
                  0, 0, 6, 16, dataStore[i].current * charinc, 2);
	XDrawString(e6display, dataStore[i].id, GC1, 0, 15,
	            dataStore[i].array, dataStore[i].end);
      }
      break;
    case  13:                /* Enter - only allowed in certain fields */
      switch ( i )  {
//      case 24:
      case 106:
        searchShow();
	break;
      default:
          XBell(e6display,20);
      }
      break;
    default:
      if ( dataStore[i].end >= dataStore[i].size )  {
        XBell(e6display, 20);
	break;
      }
      for (k=dataStore[i].end; k>dataStore[i].current; k--)  {
        dataStore[i].array[k] = dataStore[i].array[k-1];
      }
      dataStore[i].array[dataStore[i].current] = bytes[0];
      dataStore[i].current++;
      dataStore[i].end++;
      XClearWindow(e6display, dataStore[i].id);
      XCopyArea(e6display, insertMarker, dataStore[i].id, GC1,
                0, 0, 6, 16, dataStore[i].current * charinc, 2);
      XDrawString(e6display, dataStore[i].id, GC1, 0, 15,
	          dataStore[i].array, dataStore[i].end);
    }
  }
}



/*  This routine is called when the input field allows multi line input.  This
 *  processing is similar to the single line field handling but with
 *  complication added due to multi lines.  Currently only the Annotation field
 *  uses this routine.
 */

void keyNote(XKeyEvent *keyevent, int index)
{
  int     count, k, i;
  KeySym  keysymbol;
  XComposeStatus   cs;
  char    bytes[3];
//  void  drop(char *, int);

  count = XLookupString(keyevent, bytes, 3, &keysymbol, &cs);
  i = index - 1000;  /* all text input fields into here are biased by 1000 */

                      /* Determine action to be performed for each character */
//  line = 3;
  switch (count) {
  case 0:             /* Non-printable key */
   switch ( keysymbol )  {
   case  XK_Prior:         /* Page Up - out of Annotation field */
   case  XK_KP_Prior:      /* Page Up - out of Annotation field - numeric kp PgUp */
     k = dataStore[i].before;
     XWarpPointer(e6display, None, dataStore[k].id,
                  0, 0, 0, 0, dataStore[k].end*charinc, 15); 
     break;
   case  XK_Next:          /* Page Down - out of Annotation field */
   case  XK_KP_Next:       /* Page Down - out of Annotation field - numeric kp PgDn */
     k = dataStore[i].next;
     XWarpPointer(e6display, None, dataStore[k].id,
                  0, 0, 0, 0, dataStore[k].end*charinc, 15);
     break;
   case  XK_KP_Left:           /* numeric keypad Left arrow */
   case  XK_Left:              /* Left arrow */
     if ( singleLine.current <= 0 )
       XBell(e6display, 20);
     else  {
       dataStore[i].current--;
       singleLine.current--;
//       printf("left: current = %d  end = %d\n", singleLine.current, singleLine.end);
       XClearArea(e6display, dataStore[i].id, 0, line*15, 0,
	           20, False);
       XDrawString(e6display, dataStore[i].id, GC1, 0, line*15 + 15,
	            singleLine.array, singleLine.end);
       XCopyArea(e6display, insertMarker, dataStore[i].id, GC1,
                  0, 0, 6, 16, singleLine.current*charinc, line*15);
     }
     break;
   case  XK_Right:             /* Right arrow */
   case  XK_KP_Right:          /* numeric keypad Right arrow */
//       printf("right: current = %d  end = %d\n", singleLine.current, singleLine.end);
     if ( singleLine.current == singleLine.end )
       XBell(e6display, 20);
     else  {
       dataStore[i].current++;
       singleLine.current++;
       XClearArea(e6display, dataStore[i].id, 0, line*15, 0,
	           20, False);
       XDrawString(e6display, dataStore[i].id, GC1, 0, line*15 + 15,
	            singleLine.array, singleLine.end);
       XCopyArea(e6display, insertMarker, dataStore[i].id, GC1,
                  0, 0, 6, 16, singleLine.current*charinc, line*15);
     }
     break;
   case  XK_Up:                /* Up arrow */
   case  XK_KP_Up:             /* numeric keypad Up arrow */
     XBell(e6display, 20);
//     printf("Page up\n");
//     if ( line <= 1 )  XBell(e6display, 20);
//     else  {
//       line--;
//     }
     break;
   case  XK_Down:             /* Down arrow */
   case  XK_KP_Down:          /* numeric keypad Down arrow */
     XBell(e6display, 20);
//     printf("Page down\n");
//     if ( line >= 20 )  XBell(e6display, 20);
//     else  {
//       line++;
//     }
     break;
   }
   break;
  case 1:             /* Printable key */
    switch ( bytes[0] )  {
    case  8:                 /* Backspace */
//      printf("backspace: current = %d  line = %d\n", singleLine.current, line);
      if ( singleLine.current <= 0 )  {
        if ( dataStore[i].current > 0 )
          dataStore[i].current--;
	singleLine.current = 0;
        XBell(e6display, 20);
      }
      else  {
	dataStore[i].end--;
	dataStore[i].current--;
	for (k=dataStore[i].current; k<dataStore[i].end; k++)
	  dataStore[i].array[k] = dataStore[i].array[k+1];
	singleLine.end--;
	singleLine.current--;
//	printf("current = %d   end = %d\n", singleLine.current, singleLine.end);
	for (k=singleLine.current; k<singleLine.end; k++) {
	  singleLine.array[k] = singleLine.array[k+1];
//	  printf("k = %d\n", k);
	}
//	drop(singleLine.array, singleLine.end);
        XClearArea(e6display, dataStore[i].id, 0, line*15, 0,
	           20, False);
        XDrawString(e6display, dataStore[i].id, GC1, 0, line*15 + 15,
	            singleLine.array, singleLine.end);
        XCopyArea(e6display, insertMarker, dataStore[i].id, GC1,
                  0, 0, 6, 16, singleLine.current*charinc, line*15);
      }
      break;
    default:  /* print the character on the scren, if possible */
      if ( dataStore[i].end >= dataStore[i].size )  { /* enough room in store */
        XBell(e6display, 20);
	break;
      }
      if ( line >= 21 )  {   /* less than maximum line limit */
        XBell(e6display, 20);
	break;
      }
      switch ( bytes[0] )  {
      case  13:                /* Enter */
//	printf("Newline\n");
	XClearArea(e6display, dataStore[i].id, singleLine.end*charinc, line*15,
	           0, 20, False);
	line++;
	singleLine.end = 0;
	singleLine.current = 0;
        XCopyArea(e6display, insertMarker, dataStore[i].id, GC1,
                  0, 0, 6, 16, 0, line*15);
//        printf("line = %d\n", line);
        dataStore[i].array[dataStore[i].current] = '\n'; /* insert NEWLINE */
        dataStore[i].current++;
        dataStore[i].end++;
        break;
      default:  /* character not special, so insert and print character */
	for (k=dataStore[i].end; k<dataStore[i].current; k--)
	  dataStore[i].array[k] = dataStore[i].array[k-1];
	dataStore[i].array[dataStore[i].current] = bytes[0];
	dataStore[i].end++;
	dataStore[i].current++;
//	printf("current = %d   end = %d\n", singleLine.current, singleLine.end);
	for (k=singleLine.end; k>singleLine.current; k--) {
	  singleLine.array[k] = singleLine.array[k-1];
//	  printf("k = %d\n", k);
	}
	singleLine.array[singleLine.current] = bytes[0];
	singleLine.end++;
	singleLine.current++;
//	drop(singleLine.array, singleLine.end);
	XClearArea(e6display, dataStore[i].id, 0, line*15, 0,
	           20, False);
        XDrawString(e6display, dataStore[i].id, GC1, 0, line*15 + 15,
	            singleLine.array, singleLine.end);
        XCopyArea(e6display, insertMarker, dataStore[i].id, GC1,
                  0, 0, 6, 16, singleLine.current*charinc, line*15);
//	printf("line = %d  current = %d  end = %d\n",
//	    line, singleLine.current, singleLine.end);
//	drop(singleLine.array, singleLine.end);
//	drop(dataStore[i].array, dataStore[i].end);
      }
    }
  }
}






/*  This routine zeros-out all reference fields prior to them being selectively
 *  used for any particular reference type.
 */

void zeroall()
{
  int  i, j, size;

  for (i=0; i<91; i++)  {
    dataStore[i].end = 0;
    dataStore[i].current = 0;
    size = dataStore[i].size;
    for (j=0; j<size; j++)  dataStore[i].array[j] = 0;
  }
  singleLine.end = 0;
  singleLine.current = 0;
  line = 0;
}



/*  Function to redraw the text in a window which generated an expose event.
 *  The multi-line Annotation field has to be treated as a special case otherwise all
 *  lines are strung together as one after the other in a single line. */

void  replacetext(int index)
{
  int  i, j, runner, line;

  i = index - 1000;   /* all input text windows return an index bias 1000 */
  if ( i == 65 )  {  /* the Annotation field is a special case - multiple lines */
    runner = 0;
    line = 15;
    for (j=0; j<dataStore[i].end; j++)  /* march along the note text */
      if ( dataStore[i].array[j] == 10 )  { /* 'line feed' separates lines */
        XDrawString(e6display, dataStore[i].id, GC8, 0, line,
                    single, runner);
        runner = 0;
	line += 15;
      }
      else  {
        single[runner] = dataStore[i].array[j];
	runner++;
      }
    XDrawString(e6display, dataStore[i].id, GC8, 0, line,
                single, runner);
    return;
  }
  XDrawString(e6display, dataStore[i].id, GC8, 0, 15,
              dataStore[i].array, dataStore[i].end);
}


void  drop(char *array, int end)
{
  int  i;

  for (i=0; i<end; i++)  printf("%c", array[i]);
  printf("\n");
}
